#ifndef MPU9150_H
#define MPU9150_H

#include "type.h"
#include "mpu9150.h"
#include "i2c.h"
#include "SystemTickTimer.h"

extern volatile uint8_t  I2CMasterBuffer[BUFSIZE];
extern volatile uint8_t  I2CSlaveBuffer[BUFSIZE];
extern volatile uint32_t I2CReadLength, I2CWriteLength;

#define MPU6050_W		0xd0
#define MPU6050_R		0xd1
#define AK8975_W		0x18
#define AK8975_R		0x19

static uint8_t Asa[]             = {0, 0, 0};         //AK8975の感度補正データ格納用(感度補正データは工場出荷時にICに書き込まれている)
static float omega_vec[]         = {0.0, 0.0, 0.0};   //角速度ベクトル (x,y,z)[rad]
static float acc_vec[]           = {0.0, 0.0, 0.0};   //加速度ベクトル (x,y,z) [1G]
static float mag_vec[]           = {0.0, 0.0, 0.0};   //地磁気ベクトル (x,y,z) [uT]

static float omega_ref_vec[]     = {0.0,0.0,0.0};     //静止時のジャイロリファレンス
static float acc_ref_vec[]       = {0.0,0.0,0.0};     //静止時の角速度リファレンス


/*----------------------------------------------------------------------------
  Init MPU6050
 *---------------------------------------------------------------------------*/
void initMPU6050 (void)
{
	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x6B; //Address start MPU9150
	I2CMasterBuffer[2] = 0x00;
	I2CEngine();

	wait1usec(100);

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x37; //Address auxiliary I2C
	I2CMasterBuffer[2] = 0x02;
	I2CEngine();

	wait1usec(100);

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1A; //Address digital low pass filter
	I2CMasterBuffer[2] = 0x02; //100Hz
	I2CEngine();

	wait1usec(100);

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1B; //range ang
	I2CMasterBuffer[2] = 0x18; //2000deg/s
	I2CEngine();

	wait1usec(100);

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x1C; //range acc
	I2CMasterBuffer[2] = 0x18; //16g
	I2CEngine();
}
/*----------------------------------------------------------------------------
  Input  void
  Output void

  Asa[]に感度調整値を格納.
 *---------------------------------------------------------------------------*/
void initAK8975 (void)
{
	volatile uint32_t i;

	I2CWriteLength = 2;
	I2CReadLength = 3;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x10; //Address 3byte (adjust MagXYZ)
	I2CMasterBuffer[2] = AK8975_R;
	I2CEngine();

	for(i = 0;i < 3;i ++)
	{
		Asa[i] = I2CSlaveBuffer[i];
	}
}


/*----------------------------------------------------------------------------
  Input  *dest_array
  Output  void

  MPU6050のデータ (AccXYZ + GyroXYZ + temperature)
    をdest_arrayに次のように格納
  byte
  0    accX  下位
  1    accX  上位
  2    accY  下位
  3    accY  上位
  4    accZ  下位
  5    accZ  上位
  6    temp  下位
  7    temp  上位
  8    gyroX 下位
  9    gyroX　上位
  10   gyroY　下位
  11   gyroY 上位
  12   gyroZ 下位
  13   gyroZ 上位
    さらに, センサの読み取りデータを物理量に変換し, acc_vec[i]
  omega_vec[i]に格納する.
*---------------------------------------------------------------------------*/
void getDataFromMPU6050 (uint8_t *dest_array )
{
	volatile uint32_t i;
	int32_t acc[3];
	uint16_t acc_temp[3];
	int32_t omega[3];

	I2CWriteLength = 2;
	I2CReadLength = 14;
	I2CMasterBuffer[0] = MPU6050_W;
	I2CMasterBuffer[1] = 0x3B; //Address 14byte (AccXYZ + GyroXYZ + temp)
	I2CMasterBuffer[2] = MPU6050_R;
	I2CEngine();

	acc_temp[0] = (I2CSlaveBuffer[0]<<8) + I2CSlaveBuffer[1];
	acc_temp[1] = (I2CSlaveBuffer[2]<<8) + I2CSlaveBuffer[3];
	acc_temp[2] = (I2CSlaveBuffer[4]<<8) + I2CSlaveBuffer[5];

	acc_temp[0] = (~acc_temp[0]) + 1;
	acc_temp[1] = (~acc_temp[1]) + 1;
	acc_temp[2] = (~acc_temp[2]) + 1;

    //Acc
	dest_array[0]  =  0x00ff & acc_temp[0];
	dest_array[1]  = (0xff00 & acc_temp[0])>>8;
	dest_array[2]  =  0x00ff & acc_temp[1];
	dest_array[3]  = (0xff00 & acc_temp[1])>>8;
	dest_array[4]  =  0x00ff & acc_temp[2];
	dest_array[5]  = (0xff00 & acc_temp[2])>>8;
	//temparature
	dest_array[6]  = I2CSlaveBuffer[7];
	dest_array[7]  = I2CSlaveBuffer[6];
	//gyro
	dest_array[8]  = I2CSlaveBuffer[9];
	dest_array[9]  = I2CSlaveBuffer[8];
	dest_array[10] = I2CSlaveBuffer[11];
	dest_array[11] = I2CSlaveBuffer[10];
	dest_array[12] = I2CSlaveBuffer[13];
	dest_array[13] = I2CSlaveBuffer[12];

	for ( i = 0; i < BUFSIZE; i++ ) //clear I2CSlaveBuffer
	{
		I2CSlaveBuffer[i] = 0x00;
	}

	//センサ値を加速度と角速度に変換

	acc[0]   = dest_array[0] + (dest_array[1]<<8);
	if(acc[0] >= 32767) acc[0] -= 65536;
	acc_vec[0]  = ((float)acc[0] - acc_ref_vec[0])/2048.0;

	acc[1]   = dest_array[2] + (dest_array[3]<<8);
	if(acc[1] >= 32767) acc[1] -= 65536;
	acc_vec[1]  = ((float)acc[1] - acc_ref_vec[1])/2048.0;

	acc[2]   = dest_array[4] + (dest_array[5]<<8);
	if(acc[0] >= 32767) acc[2] -= 65536;
	acc_vec[2]  = ((float)acc[2] - acc_ref_vec[2])/2048.0;

	omega[0]   = dest_array[8] + (dest_array[9]<<8);
	if(omega[0] >= 32767) omega[0] -= 65536;
	omega_vec[0]  = ((float)omega[0] - omega_ref_vec[0])/16.4;

	omega[1]   = dest_array[10] + (dest_array[11]<<8);
	if(omega[1] >= 32767) omega[1] -= 65536;
	omega_vec[1]  = ((float)omega[1] - omega_ref_vec[1])/16.4;

	omega[2]   = dest_array[12] + (dest_array[13]<<8);
	if(omega[2] >= 32767) omega[2] -= 65536;
	omega_vec[2]  = ((float)omega[2] - omega_ref_vec[2])/16.4;



	return;
}
/*----------------------------------------------------------------------------
  Input   *dest_array
  Output  void

  AK8975のデータ(MagXYZ)
    をdest_arrayに次のように格納
  Asa[]: 工場出荷時にAK8975に書きこまれた補正値
  AK8975の初期化の際にAsa[]に格納

 byte
  0    MagX  下位
  1    MagX  上位
  2    MagY  下位
  3    MagY  上位
  4    MagZ  下位
  5    MagZ  上位
    さらにセンサの読み取り値を物理量に変換しmag_vec[i]に格納する.
 *---------------------------------------------------------------------------*/
void getDataFromAK8975 (uint8_t *dest_array)
{
	float mag[3];

	volatile uint32_t i;
	int16_t Mag[3];

	I2CWriteLength = 2;
	I2CReadLength = 6;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x03; //Address 6byte (MagXYZ)
	I2CMasterBuffer[2] = AK8975_R;
	I2CEngine();

	for ( i = 0; i < 3; i++ )
	{
		Mag[i] = (uint16_t)I2CSlaveBuffer[i<<1] | (((uint16_t)I2CSlaveBuffer[(i<<1)+1])<<8);
		Mag[i] = Mag[i]*(uint16_t)((((int8_t)Asa[i] - 128)/256) + 1);
	}

	dest_array[0] = (uint8_t)Mag[1];
	dest_array[1] = (uint8_t)(Mag[1]>>8);

	dest_array[2] = (uint8_t)Mag[0];
	dest_array[3] = (uint8_t)(Mag[0]>>8);

	Mag[2] = (~Mag[2]) + 1; //地磁気センサのZ軸の取り方を下向きから上向きに変更
	dest_array[4] = (uint8_t)Mag[2];
	dest_array[5] = (uint8_t)(Mag[2]>>8);

	wait1usec(100);

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = AK8975_W;
	I2CMasterBuffer[1] = 0x0A; //Address start MagXYZ ADC (need 7.2ms)
	I2CMasterBuffer[2] = 0x01;
	I2CEngine();

	for ( i = 0; i < BUFSIZE; i++ ) //clear I2CSlaveBuffer
	{
		I2CSlaveBuffer[i] = 0x00;
	}

	//センサ値を地磁気ベクトルに変換
	for( i=0;i<3;i++)
	{
		mag[i]   = (float)(dest_array[i] + dest_array[i+1]<<8);
		if(mag[i] >= 32767.0) mag[i] -= 65535.0;
		mag_vec[i]  = mag[i]*0.3;
	}

	return ;
}


void getOmega(float *vec)
{
	uint8_t i;
	for(i=0;i<3;i++) vec[i] = omega_vec[i];
	return;
};
void getMag(float *vec)
{
	uint8_t i;
	for(i=0;i<3;i++) vec[i] = mag_vec[i];
	return;
};
void getAcc(float *vec)
{
	uint8_t i;
	for(i=0;i<3;i++) vec[i] = acc_vec[i];
	return;
};

#endif

